<?php
// The router
if (preg_match('/\.(?:png|jpg|jpeg|gif)$/', $_SERVER["REQUEST_URI"])) {
    return false;    // serve the requested resource as-is.
}

require_once ("vendor/autoloader.php");
require_once ("src/Fail2ban.php");
require_once ("src/Configuration.php");

// The three following are used to fake the rewrite mode
if (strpos($_SERVER["REQUEST_URI"], "/index.php?url=") === false)
  $_SERVER["REQUEST_URI"] = "/index.php?url=".substr ($_SERVER["REQUEST_URI"], 1);
$_SERVER["SCRIPT_NAME"] = "/index.php";

$configuration = new Configuration();
if (! isset ($_SERVER["HTTP_AUTHORIZATION"]) ||
  $_SERVER["HTTP_AUTHORIZATION"] !== "Bearer " . $configuration->getSharedKey())
  die (http_response_code (401));

$route = new Domframework\Route();
$route->ratelimiter->storageDir = "/tmp/fail2banmgr-server";
$route->output = "rest";

$fail2ban = new Fail2ban();
if ($configuration->getF2BLogsDir())
  $fail2ban->setF2BLogsDir($configuration->getF2BLogsDir());
if ($configuration->getF2BClient())
  $fail2ban->setF2BClient($configuration->getF2BClient());

$route
->get("jails/{jail}/logs/{ip}", function($jail, $ip) use ($fail2ban) {
  return $fail2ban->getLogsIPinJail($jail, $ip);
})
->post("jails/{jail}/ips/{ip}", function($jail, $ip) use ($fail2ban) {
  http_response_code(201);
  return $fail2ban->addIPinJail($jail, $ip);
})
->delete("jails/{jail}/ips/{ip}", function($jail, $ip) use ($fail2ban) {
  return $fail2ban->delIPinJail($jail, $ip);
})
->get("jails/{jail}/ips", function($jail) use ($fail2ban) {
  return $fail2ban->getJailedIPs($jail);
})
->get("jails/{jail}/logs", function($jail) use ($fail2ban) {
  return $fail2ban->getJailedIPsLogs($jail);
})
->get("jails", function() use ($fail2ban) {
  return $fail2ban->getJails();
})
->get("synthesis", function() use ($fail2ban) {
  return $fail2ban->synthesis();
})
->multi("get,post,put,delete", "({content})?", function ($content) {
  throw new \Exception ("Page Not Found", 404);
})
;
