<?php
class Configuration
{
  private $confFile;
  private $content;

  public function __construct($confFile = "/etc/fail2banmgr-server.ini")
  {
    if (! file_exists ($confFile))
      @file_put_contents ($confFile, "[global]\nsharedKey = ".sha1(time())."\n");
    if (! file_exists ($confFile))
      throw new \Exception("Fail2ban Manager configuration not creatable '$confFile'", 500);
    if (! is_readable ($confFile))
      throw new \Exception("Fail2ban Manager configuration not readable '$confFile'", 500);
    $this->confFile = $confFile;
    $this->content = parse_ini_file ($this->confFile, true);
  }

  public function getSharedKey()
  {
    if (! isset($this->content["global"]["sharedKey"]))
      throw new \Exception("Fail2ban Manager : no sharedkey in global configuration", 500);
    return $this->content["global"]["sharedKey"];
  }

  public function getF2BClient()
  {
    if (! isset($this->content["global"]["fail2banClient"]))
      return null;
    return $this->content["global"]["fail2banClient"];
  }

  public function getF2BLogsDir()
  {
    if (! isset($this->content["global"]["fail2banLogsDir"]))
      return null;
    return $this->content["global"]["fail2banLogsDir"];
  }
}
