<?php
// Fail2ban library
// Need exec as the protocol in the fail2ban socket need python pickle
// serializer

class Fail2ban
{
    private $fail2banClient = "/usr/bin/fail2ban-client";
    private $fail2banLogsDir = "/tmp";

    public function __construct()
    {
        if (! function_exists("exec"))
            throw new \Exception("Fail2ban can not use 'exec' : not enabled in PHP",
            500);
        if (posix_geteuid() !== 0)
            throw new \Exception("Fail2ban must be run by root", 500);
    }

    public function setFail2banClient(string $fail2banClient):self
    {
        $this->fail2banClient = $fail2banClient;

        return $this;
    }

    public function setFail2banLogsDir(string $fail2banLogsDir):self
    {
        $this->fail2banLogsDir = $fail2banLogsDir;

        return $this;
    }

    public function getJails(): array
    {
        if (! file_exists($this->fail2banClient) ||
            ! is_executable($this->fail2banClient))
            throw new \Execption ($this->fail2banClient . " is not installed", 500);
        exec($this->fail2banClient." status 2>/dev/null", $output, $rc);
        $jails = preg_grep ("#Jail list:#", $output);
        if (empty ($jails))
            return [];
        // Jail format : Jail list:\trecidive, sshd, zimbra-proxy, zimbra-proxyv2
        $jails = reset ($jails);
        $jails = substr (strstr($jails, "\t"), 1);
        if ($jails === false || $jails === "")
            return [];
        return explode (", ", $jails);
    }

    public function getJailedIPs(string $jail): array
    {
        if (! file_exists($this->fail2banClient) ||
            ! is_executable($this->fail2banClient))
            throw new \Execption ($this->fail2banClient . " is not installed", 500);
        exec($this->fail2banClient." status ".escapeshellarg ($jail)." 2>/dev/null",
            $output, $rc);
        $ips = preg_grep ("#Banned IP list:#", $output);
        if (empty ($ips))
            throw new \Exception("Fail2ban : invalid jail provided", 406);
        // Format : Banned IP list:	37.170.202.161 37.172.60.37 37.169.12.118
        $ips = reset ($ips);
        $ips = substr (strstr($ips, "\t"), 1);
        if ($ips === false || $ips === "")
            return [];
        $ips = explode (" ", $ips);
        return $ips;
    }

    public function getLogsIPinJail(string $jail, string $ip): string
    {
        $ipaddresses = new Domframework\Ipaddresses();
        if (! $ipaddresses->validIPAddress($ip))
            throw new \Exception("Fail2ban : invalid IP address provided", 406);
        $logFile = $this->fail2banLogsDir."/fail2banip-$jail-$ip";
        if (! file_exists ($logFile))
            throw new \Exception("Fail2ban : Can not found the complain logs for the provided Jail and IP", 404);
        return file_get_contents($logFile);
    }
    public function addIPinJail(string $jail, string $ip): bool
    {
        if (! file_exists($this->fail2banClient) ||
            ! is_executable($this->fail2banClient))
            throw new \Execption ($this->fail2banClient . " is not installed", 500);
        $ipaddresses = new Domframework\Ipaddresses();
        if (! $ipaddresses->validIPAddress($ip))
            throw new \Exception("Fail2ban : invalid IP address provided", 406);
        exec($this->fail2banClient." set ".escapeshellarg ($jail)." banip ".
            escapeshellarg ($ip)." 2>/dev/null",
            $output, $rc);
        if ($rc === 0)
            return true;
        throw new \Exception("Fail2ban : invalid jail provided", 406);
    }

    public function delIPinJail(string $jail, string $ip): bool
    {
        if (! file_exists($this->fail2banClient) ||
            ! is_executable($this->fail2banClient))
            throw new \Execption ($this->fail2banClient . " is not installed", 500);
        $ipaddresses = new Domframework\Ipaddresses();
        if (! $ipaddresses->validIPAddress($ip))
            throw new \Exception("Fail2ban : invalid IP address provided", 406);
        exec($this->fail2banClient." set ".escapeshellarg ($jail)." unbanip ".
            escapeshellarg ($ip)." 2>/dev/null",
            $output, $rc);
        if ($rc === 0)
            return true;
        if (strpos(reset($output), "is not banned") !== false)
            throw new \Exception("Fail2ban : IP address to unban was not already banned", 406);
        throw new \Exception("Fail2ban : invalid jail provided", 406);
    }
    public function getJailedIPsLogs(string $jail): array
    {
        if (! in_array($jail, $this->getJails()))
            throw new \Exception("Fail2banmgr : jail '$jail' doesn't exists", 404);
        $files = glob($this->fail2banLogsDir."/fail2banip-$jail-*");
        foreach($files as &$file)
        {
            $file = substr(strrchr($file, "-"), 1);
        }
        return $files;
    }

    public function synthesis(): array
    {
        $res = [];
        foreach ($this->getJails() as $jail) {
            $logs = $this->getJailedIPsLogs($jail);
            foreach ($this->getJailedIPs($jail) as $bannedIP) {
                $res[] = [
                    "jail" => $jail,
                    "bannedIP" => $bannedIP,
                    "log" => in_array($bannedIP, $logs),
                ];
            }
        }
        return $res;
    }
}
